use crate::constant::{MESSAGE_KIND_CREATE_LOBBY, MESSAGE_KIND_CREATE_PLAYER, MESSAGE_KIND_MOVE, METADATA_SIZE};

// everything here needs to be adjusted for the deserialized json
//
#[derive(Debug, Clone)]
pub struct Message {
    pub message_kind: MessageKind,
    pub length: u16,
    pub content: String,
}

// idk i just want to layout something, really didn't think much
#[derive(Debug, Clone)]
pub enum MessageKind {
    Move,
    CreatePlayer,
    CreateLobby,
}

impl Message {
    pub fn new(message_kind: MessageKind, content: impl Into<String>) -> anyhow::Result<Self> {
        let content = content.into();
        let length = content.len() as u16;

        Ok(Self {
            message_kind,
            length,
            content,
        })
    }

    pub fn encode(&self) -> Vec<u8> {
        let mut buffer = Vec::new();
        buffer.extend(&self.length.to_be_bytes());
        buffer.extend(self.content.as_bytes());

        buffer
    }

    pub fn decode(buffer: &[u8]) -> anyhow::Result<Self> {
        if buffer.len() < METADATA_SIZE {
            return Err(anyhow::anyhow!("Invalid message length"));
        }

        let message_kind = MessageKind::try_from(buffer[0]).expect("Error parsing the MessageKind");
        let length = u16::from_be_bytes([buffer[1], buffer[2]]);
        let content = String::from_utf8(buffer[3..3 + length as usize].to_vec())?;

        Ok(Self {
            message_kind,
            length,
            content,
        })
    }
}

impl TryFrom<u8> for MessageKind {
    type Error = &'static str;

    fn try_from(value: u8) -> Result<Self, Self::Error> {
        match value {
            MESSAGE_KIND_MOVE => Ok(MessageKind::Move),
            MESSAGE_KIND_CREATE_PLAYER => Ok(MessageKind::CreatePlayer),
            MESSAGE_KIND_CREATE_LOBBY => Ok(MessageKind::CreateLobby),
            _ => Err("invalid MessageKind byte")
        }
    }
}
